<?php

namespace Drupal\dynamic_tasks\Form;

use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides dynamic tasks settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The dynamic tasks settings name.
   */
  const SETTINGS = 'dynamic_tasks.settings';

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected ExtensionPathResolver $extensionPathResolver;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): SettingsForm {
    $instance = parent::create($container);
    $instance->extensionPathResolver = $container->get('extension.path.resolver');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'dynamic_tasks_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [static::SETTINGS];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config(static::SETTINGS);
    $this->messenger()->addWarning($this->t('Caches are flushed upon save for the changes to take effect.'));
    $form['tasks'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Tasks (YAML)'),
      '#description' => $this->t('Define local tasks using YAML. See @link for more information.', [
        '@link' => Link::fromTextAndUrl('https://www.drupal.org/docs/drupal-apis/menu-api/providing-module-defined-local-tasks',
          Url::fromUri('https://www.drupal.org/docs/drupal-apis/menu-api/providing-module-defined-local-tasks')
        )->toString(),
      ]),
      '#default_value' => $config->get('tasks'),
      '#attributes' => ['data-yaml-editor' => 'true'],
    ];
    $form['help'] = [
      '#type' => 'details',
      '#title' => $this->t('Help'),
    ];
    $form['help']['example'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Example: Tasks (YAML)'),
      '#description' => $this->t('Add secondary local tasks to the user canonical route.'),
      '#disabled' => TRUE,
      '#default_value' => file_get_contents($this->extensionPathResolver->getPath('module', 'dynamic_tasks') . '/config/schema/schema.example.yml'),
      '#attributes' => ['data-yaml-editor' => 'true'],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable(static::SETTINGS);
    $config->set('tasks', $form_state->getValue('tasks'));
    $config->save();
    drupal_flush_all_caches();
    parent::submitForm($form, $form_state);
  }

}
